==> readstore_basic-1.3.1.tar.gz: readstore_basic-1.3.1/readstore_basic/backend/app/models.py <==
# readstore-basic/backend/app/models.py

import string
import uuid
import base64
import random

from django.db import models
from django.contrib.auth.models import User


def generate_short_uuid() -> str:
    """Generate short uuid.

    Generate a short uuid.
    Length is defined to 22 chars.
    UUIDs contain only alphanumeric characters.
    
    Returns:
        (str): short uuid
    """
    
    ALNUM_CHARS = string.ascii_letters + string.digits
    
    # Generate a UUID4
    uuid_bytes = uuid.uuid4().bytes
    
    # Encode the UUID bytes to base64
    base64_uuid = base64.urlsafe_b64encode(uuid_bytes)
    
    # Decode base64 bytes to string and remove padding characters
    short_uuid = base64_uuid.decode('utf-8').rstrip('=')
    
    # Remove any underscores and dashes by random char
    short_uuid = short_uuid.replace('_',
                                    ALNUM_CHARS[random.randint(0, len(ALNUM_CHARS) - 1)])
    short_uuid = short_uuid.replace('-',
                                    ALNUM_CHARS[random.randint(0, len(ALNUM_CHARS) - 1)])

    return short_uuid


# Extend User class for project
class AppUser(models.Model):
    """
        Profile for User Model
    """
    
    user = models.OneToOneField(User, related_name='appuser', on_delete=models.CASCADE, null=True, blank=True)
    token = models.CharField(max_length=200, default=generate_short_uuid)
    owner_group = models.ForeignKey('OwnerGroup', on_delete=models.PROTECT, null=True)
    
    def __str__(self):
        return self.user.username
    
    def regenerate_token(self):
        self.token = generate_short_uuid()
        self.save()
    
# Abstract models for Django
class BasicModel(models.Model):  
    """
        Basic Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    valid_from = models.DateTimeField(auto_now=True)
    valid_to = models.DateTimeField(null=True, blank=True)
    owner = models.ForeignKey(User, on_delete=models.CASCADE)
    
    class Meta:
        abstract = True

class PipelineModel(BasicModel):
    """
        Pipeline Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    pipeline_version = models.TextField()
    
    class Meta:
        abstract = True

class BasicFileModel(BasicModel):
    
    """
        BasicFileModel extension for BasicModel
        Add default fields generally used for file definitions
    """
    
    name = models.TextField(unique=True) # Needs to be updated
    description = models.TextField(blank=True, null=True)
    path = models.TextField()
    filetype = models.CharField(max_length=150)
    size_mb = models.IntegerField()
    
    class Meta:
        abstract = True

class FlatFileModel(BasicFileModel):
    
    """
        Flat File Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    body = models.TextField()
    
    class Meta:
        abstract = True

class BinaryFileModel(BasicFileModel):
    
    """
        Flat File Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    body = models.BinaryField()
    
    class Meta:
        abstract = True
        
class ObjectStoreModel(BasicFileModel):
    
    """
        Object Store Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    s3_bucket = models.TextField()
    s3_key = models.TextField()
    
    class Meta:
        abstract = True
    
# MODELS

class OwnerGroup(BasicModel):
    
    """
        OwnerGroup Model
    """
    
    name = models.CharField(max_length=200, unique=True)
    owner = owner = models.ForeignKey(User, on_delete=models.PROTECT)
    
    class Meta:
        db_table = 'owner_group'


class FqFile(PipelineModel):
    
    """
        FqFile Model to manage uploaded files
        with S3 bucket and key fields
    """
    
    class ReadType(models.TextChoices):
        READ1 = 'R1'
        READ2 = 'R2'
        INDEX1 = 'I1'
        INDEX2 = 'I2'
        NA = 'NA'
    
    name = models.TextField()
    bucket = models.TextField(blank=True, null=True)
    key = models.TextField(blank=True, null=True)
    upload_path = models.TextField()
    qc_passed = models.BooleanField()
    read_type = models.CharField(choices=ReadType.choices, default=ReadType.NA, max_length=20)
    read_length = models.IntegerField()
    num_reads = models.IntegerField()
    qc_phred_mean = models.FloatField()
    qc_phred = models.JSONField() 
    size_mb = models.IntegerField()
    staging = models.BooleanField()
    md5_checksum = models.TextField()
    
    class Meta:
        db_table = 'fq_file'
        #unique_together = ('bucket', 'key')
        
class Project(BasicModel):
    
    """
        Project Model
    """
    
    name = models.CharField(max_length=200)
    description = models.TextField(null=True, blank=True)
    metadata  = models.JSONField()
    dataset_metadata_keys = models.JSONField()
    collaborators = models.ManyToManyField(User, related_name='projects', blank=True)
    owner_group = models.ForeignKey(OwnerGroup, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'project'
        unique_together = ('name', 'owner_group')
        
                
class ProjectAttachment(BinaryFileModel):
    
    """
        Project Attachments to store binary files. 
    """
    
    name = models.TextField()
    project = models.ForeignKey(Project, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'project_attachment'
        unique_together = ('name', 'project')
       

class FqDataset(BasicModel):
    
    """
        Model to organize FqFiles and metadata
    """
    
    name = models.TextField()
    description = models.TextField(null=True, blank=True)
    qc_passed = models.BooleanField()
    fq_file_r1 = models.OneToOneField(FqFile, related_name='fq_file_r1', on_delete=models.SET_NULL, null=True, blank=True)
    fq_file_r2 = models.OneToOneField(FqFile, related_name='fq_file_r2', on_delete=models.SET_NULL, null=True, blank=True)
    fq_file_i1 = models.OneToOneField(FqFile, related_name='fq_file_i1', on_delete=models.SET_NULL, null=True, blank=True)
    fq_file_i2 = models.OneToOneField(FqFile, related_name='fq_file_i2', on_delete=models.SET_NULL, null=True, blank=True)
    paired_end = models.BooleanField()
    index_read = models.BooleanField()
    owner_group = models.ForeignKey(OwnerGroup, on_delete=models.CASCADE)
    project = models.ManyToManyField(Project, related_name='fq_datasets', blank=True)
    metadata = models.JSONField()

    class Meta:
        db_table = 'fq_dataset'
        unique_together = ('name', 'owner_group')


class FqAttachment(BinaryFileModel):
    
    """
        Model to store binary files for FqDatasets. 
    """
    
    name = models.TextField()
    fq_dataset = models.ForeignKey(FqDataset, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'fq_attachment'
        unique_together = ('name', 'fq_dataset')
        
class LicenseKey(BasicModel):
    
    """
        LicenseKeys Model
    """
    
    key = models.TextField()
    seats = models.IntegerField()
    expiration_date = models.DateTimeField()
    
    class Meta:
        db_table = 'license_keys'
    
class ProData(BasicModel):
    
    """
        ProData Model
    """
    
    name = models.TextField()
    data_type = models.TextField()
    description = models.TextField(blank=True, null=True)
    version = models.IntegerField()
    upload_path = models.TextField()
    metadata = models.JSONField()
    fq_dataset = models.ForeignKey(FqDataset, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'pro_data'
        unique_together = ('name', 'version', 'fq_dataset')

==> readstore_basic-1.4.1.tar.gz: readstore_basic-1.4.1/readstore_basic/backend/app/models.py <==
# readstore-basic/backend/app/models.py

import string
import uuid
import base64
import random

from django.db import models
from django.contrib.auth.models import User
from django.dispatch import receiver

#region Functions

def generate_short_uuid() -> str:
    """Generate short uuid.

    Generate a short uuid.
    Length is defined to 22 chars.
    UUIDs contain only alphanumeric characters.
    
    Returns:
        (str): short uuid
    """
    
    ALNUM_CHARS = string.ascii_letters + string.digits
    
    # Generate a UUID4
    uuid_bytes = uuid.uuid4().bytes
    
    # Encode the UUID bytes to base64
    base64_uuid = base64.urlsafe_b64encode(uuid_bytes)
    
    # Decode base64 bytes to string and remove padding characters
    short_uuid = base64_uuid.decode('utf-8').rstrip('=')
    
    # Remove any underscores and dashes by random char
    short_uuid = short_uuid.replace('_',
                                    ALNUM_CHARS[random.randint(0, len(ALNUM_CHARS) - 1)])
    short_uuid = short_uuid.replace('-',
                                    ALNUM_CHARS[random.randint(0, len(ALNUM_CHARS) - 1)])

    return short_uuid


# Extend User class for project
class AppUser(models.Model):
    """
        Profile for User Model
    """
    
    user = models.OneToOneField(User, related_name='appuser', on_delete=models.CASCADE, null=True, blank=True)
    token = models.CharField(max_length=200, default=generate_short_uuid)
    owner_group = models.ForeignKey('OwnerGroup', on_delete=models.PROTECT, null=True)
    
    def __str__(self):
        return self.user.username
    
    def regenerate_token(self):
        self.token = generate_short_uuid()
        self.save()
    
# Abstract models for Django
class BasicModel(models.Model):  
    """
        Basic Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    valid_from = models.DateTimeField(auto_now=True)
    valid_to = models.DateTimeField(null=True, blank=True)
    owner = models.ForeignKey(User, on_delete=models.CASCADE)
    
    class Meta:
        abstract = True

class PipelineModel(BasicModel):
    """
        Pipeline Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    pipeline_version = models.TextField()
    
    class Meta:
        abstract = True

class BasicFileModel(BasicModel):
    
    """
        BasicFileModel extension for BasicModel
        Add default fields generally used for file definitions
    """
    
    name = models.TextField(unique=True) # Needs to be updated
    description = models.TextField(blank=True, null=True)
    path = models.TextField()
    filetype = models.CharField(max_length=150)
    size_mb = models.IntegerField()
    
    class Meta:
        abstract = True

class FlatFileModel(BasicFileModel):
    
    """
        Flat File Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    body = models.TextField()
    
    class Meta:
        abstract = True

class BinaryFileModel(BasicFileModel):
    
    """
        Flat File Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    body = models.BinaryField()
    
    class Meta:
        abstract = True
        
class ObjectStoreModel(BasicFileModel):
    
    """
        Object Store Model extension for Django models.Model
        Add default fields generally used in Evo Models
    """
    
    s3_bucket = models.TextField()
    s3_key = models.TextField()
    
    class Meta:
        abstract = True
    
# MODELS

class OwnerGroup(BasicModel):
    
    """
        OwnerGroup Model
    """
    
    name = models.CharField(max_length=200, unique=True)
    owner = owner = models.ForeignKey(User, on_delete=models.PROTECT)
    
    class Meta:
        db_table = 'owner_group'

class AttachmentBody(models.Model):
    
    """
        AttachmentBody Model to store binary files. 
    """
    
    body = models.BinaryField()
    
    class Meta:
        db_table = 'attachment_body'

class FqFile(PipelineModel):
    
    """
        FqFile Model to manage uploaded files
        with S3 bucket and key fields
    """
    
    class ReadType(models.TextChoices):
        READ1 = 'R1'
        READ2 = 'R2'
        INDEX1 = 'I1'
        INDEX2 = 'I2'
        NA = 'NA'
    
    name = models.TextField()
    bucket = models.TextField(blank=True, null=True)
    key = models.TextField(blank=True, null=True)
    upload_path = models.TextField()
    qc_passed = models.BooleanField()
    read_type = models.CharField(choices=ReadType.choices, default=ReadType.NA, max_length=20)
    read_length = models.IntegerField()
    num_reads = models.IntegerField()
    qc_phred_mean = models.FloatField()
    qc_phred = models.JSONField() 
    size_mb = models.IntegerField()
    staging = models.BooleanField()
    md5_checksum = models.TextField()
    
    class Meta:
        db_table = 'fq_file'
        #unique_together = ('bucket', 'key')
    
    def has_fq_dataset(self) -> bool:
        """Check if FqFile is associated with FqDataset.

        Check if FqFile is associated with FqDataset via OneToOneField.
        
        Returns:
            bool: True if FqFile is associated with FqDataset, False otherwise
        """
        fq_attributes = ['fq_file_r1', 'fq_file_r2', 'fq_file_i1', 'fq_file_i2']
        for fq_attribute in fq_attributes:
            if hasattr(self, fq_attribute):
                return True
        else:
            return False
    
    def get_fq_dataset_id(self) -> int | None:
        """Get FqDataset ID.

        Get FqDataset ID associated with FqFile.
        
        Returns:
            int: FqDataset ID
        """
        
        fq_attributes = ['fq_file_r1', 'fq_file_r2', 'fq_file_i1', 'fq_file_i2']
        for fq_attribute in fq_attributes:
            if hasattr(self, fq_attribute):
                return getattr(self, fq_attribute).id
        else:
            return None


class Project(BasicModel):
    
    """
        Project Model
    """
    
    name = models.CharField(max_length=200, unique=True)
    description = models.TextField(null=True, blank=True)
    metadata  = models.JSONField()
    dataset_metadata_keys = models.JSONField()
    collaborators = models.ManyToManyField(User, related_name='projects', blank=True)
    owner_group = models.ForeignKey(OwnerGroup, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'project'        
            
            
class ProjectAttachment(BasicFileModel):
    """
        Project Attachments to store binary files. 
    """
    
    name = models.TextField()
    attachment_body = models.OneToOneField(AttachmentBody, on_delete=models.PROTECT)   
    project = models.ForeignKey(Project, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'project_attachment'
        unique_together = ('name', 'project')

@receiver(models.signals.post_delete, sender=ProjectAttachment)
def delete_project_attachment_bodies(sender, instance, **kwargs):
    """
        Delete FqAttachmentBodies
    """
    
    instance.attachment_body.delete()

class FqDataset(BasicModel):
    
    """
        Model to organize FqFiles and metadata
    """
    
    name = models.TextField(unique=True)
    description = models.TextField(null=True, blank=True)
    qc_passed = models.BooleanField()
    fq_file_r1 = models.OneToOneField(FqFile, related_name='fq_file_r1', on_delete=models.SET_NULL, null=True, blank=True)
    fq_file_r2 = models.OneToOneField(FqFile, related_name='fq_file_r2', on_delete=models.SET_NULL, null=True, blank=True)
    fq_file_i1 = models.OneToOneField(FqFile, related_name='fq_file_i1', on_delete=models.SET_NULL, null=True, blank=True)
    fq_file_i2 = models.OneToOneField(FqFile, related_name='fq_file_i2', on_delete=models.SET_NULL, null=True, blank=True)
    paired_end = models.BooleanField()
    index_read = models.BooleanField()
    owner_group = models.ForeignKey(OwnerGroup, on_delete=models.CASCADE)
    project = models.ManyToManyField(Project, related_name='fq_datasets', blank=True)
    metadata = models.JSONField()

    class Meta:
        db_table = 'fq_dataset'
        

class FqAttachment(BasicFileModel):
    
    """
        Model to store binary files for FqDatasets. 
    """
    
    name = models.TextField()
    attachment_body = models.OneToOneField(AttachmentBody, on_delete=models.PROTECT)
    fq_dataset = models.ForeignKey(FqDataset, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'fq_attachment'
        unique_together = ('name', 'fq_dataset')


@receiver(models.signals.post_delete, sender=FqAttachment)
def delete_fq_attachment_bodies(sender, instance, **kwargs):
    """
        Delete FqAttachmentBodies
    """
    
    instance.attachment_body.delete()

        
class LicenseKey(BasicModel):
    
    """
        LicenseKeys Model
    """
    
    key = models.TextField()
    seats = models.IntegerField()
    expiration_date = models.DateTimeField()
    
    class Meta:
        db_table = 'license_keys'
    
class ProData(BasicModel):
    
    """
        ProData Model
    """
    
    name = models.TextField()
    data_type = models.TextField()
    description = models.TextField(blank=True, null=True)
    version = models.IntegerField()
    upload_path = models.TextField()
    metadata = models.JSONField()
    fq_dataset = models.ForeignKey(FqDataset, on_delete=models.CASCADE)
    
    class Meta:
        db_table = 'pro_data'
        unique_together = ('name', 'version', 'fq_dataset')
